package org.greenstone.gsdl3.gs3build.util;



import java.net.*;

import java.io.*;



public class HTMLLoader implements Runnable

{ URL 					from;

	StringBuffer	HTML;

	boolean				loaded;

	boolean				error;

	boolean				notFound;

	Thread				thread;

	static int		running;



	/**

	 * Constructor to load given URL

	 */

	public HTMLLoader(URL from)

	{ this.from 	= from;

		this.HTML 	= new StringBuffer();

		this.loaded	= false;

		this.error	= false;

		this.thread = null;
	}



	/**

	 * @return <code>0</code> if finished running; any other value if

	 *         still executing

	 */

	public static synchronized int running()

	{ return running;

	}



	/**

	 * Starts the thread; the method <code>load</code> is a synonym for this

	 */

	public synchronized void start()

	{ if (thread == null)

		{ thread = new Thread(this);

			thread.start();

			running	++;

		}

//		System.out.println("Running: "+running);

	}



	/**

	 * Do the actual loading; this will multi-thread with calling task

	 */

	public void run()

	{	URLConnection connect;

		int						length, i, c;

		byte					inbuffer[];

		InputStream		in;



		inbuffer = new byte[128];



		this.notFound	= false;
//		System.out.println("Running loader");



		// get connection and test length

		try

		{	connect = this.from.openConnection();

		}

		catch (IOException ioex)

		{ this.loaded	= true;

			this.error	=	true;

			running --;
			System.out.println(ioex);

			return;

		}


		length = connect.getContentLength();

		if (length == 0)

		{ this.loaded = true;

			running --;
			System.out.println("Empty");

			return;

		}



		// get an inputstream to the document body

		try

		{ in = connect.getInputStream();

		}

		catch (IOException ioex)

		{ this.error	= true;

			this.loaded	= true;

			running --;
			System.out.println("Open failed" + ioex);

			return;

		}



		if (length < 0)

		{	if (connect.getHeaderField(0) != null &&

					connect.getHeaderField(0).indexOf("404") >= 0)

			{	this.notFound	= true;

				this.error		= true;

				this.loaded		= true;

				running --;
				System.out.println("Not found");

				return;

			}

		}

//		System.out.println("Ready to read " + length);



		// read in the document body

		i = 0;

		c = 0;

		while (true || i < length)

		{ // read in 128 characters (abort on IO Exception)

			try

			{ c = in.read(inbuffer);

			}

			catch (IOException ioex)

			{ break;

			}

			// System.out.println("read " + c);



			// if no characters were read, end, otherwise add it to the body

			if (c != -1)

			{ if (c > 0)

				{ this.HTML.append(new String(inbuffer, 0, c));

					i += c;

				}

			}

			else

			{ break;

			}

			// if an incomplete buffer was read (ie no characters remain), end

/*			if (c < inbuffer.length && i >= length)

			{ break;

			}*/

		}



		// Close file

		try

		{	in.close();

		}

		catch (IOException ioex)

		{	//System.out.println("Closed except "+ioex);

		}

	
		this.loaded = true;

  	running --;

	}



	/**

	 * Do the actual loading (calls a Start on the Thread)

	 */

	public void load()

	{	this.start();

//		this.run();

	}



	/**

	 * @return HTML text as a <code>String</code>.

	 */

	public synchronized String HTML()

	{ if (this.error)

		{ return "Error";

		}

		return this.HTML.toString();

	}



	/**

	 * @return  <code>true</code> if the file has successfully loaded

	 */

	public boolean loaded()

	{ return this.loaded;

	}



	protected void finalize() throws Throwable

	{	this.thread = null;

		this.from		= null;

		this.HTML		= null;

		super.finalize();

	}



	static

	{ running = 0;

	}

}